@@ -1,15 +1,22 @@
 auto.arima <- function(x, d=NA, D=NA, max.p=5, max.q=5,
     max.P=2, max.Q=2, max.order=5,
     start.p=2, start.q=2, start.P=1, start.Q=1,
-    stationary=FALSE, ic=c("aic","aicc","bic"),
+    stationary=FALSE, ic=c("aicc","aic","bic"),
     stepwise=TRUE, trace=FALSE,
     approximation=(length(x)>100 | frequency(x)>12), xreg=NULL,
-		test=c("kpss","adf","pp"), seasonal.test=c("ocsb","ch"), 
-		allowdrift=TRUE,lambda=NULL)
+    test=c("kpss","adf","pp"), seasonal.test=c("ocsb","ch"), 
+    allowdrift=TRUE,lambda=NULL,
+#########################################################################
+        parallel=FALSE, num.cores=NULL)
 {
+        if (stepwise==TRUE & parallel==TRUE){
+            warning("Parallel computer is only implemented when stepwise=FALSE, the model will be fit in serial.")
+			parallel <- FALSE
+        }
+#########################################################################
     ic <- match.arg(ic)
     test <- match.arg(test)
-		seasonal.test <- match.arg(seasonal.test)
+        seasonal.test <- match.arg(seasonal.test)
     m <- frequency(x)
 
     if(m < 1)
@@ -17,10 +24,14 @@ auto.arima <- function(x, d=NA, D=NA, max.p=5, max.q=5,
         warning("I can't handle data with frequency less than 1. Seasonality will be ignored.")
         m <- 1
     }
+	max.p<-ifelse(max.p <= floor(length(x)/3), max.p, floor(length(x)/3))
+	max.q<-ifelse(max.q <= floor(length(x)/3), max.q, floor(length(x)/3))
+	max.P<-ifelse(max.P <= floor((length(x)/3)/m), max.P, floor((length(x)/3)/m))
+	max.Q<-ifelse(max.Q <= floor((length(x)/3)/m), max.Q, floor((length(x)/3)/m))
 
-		orig.x <- x
-		if(!is.null(lambda))
-			x <- BoxCox(x,lambda)
+        orig.x <- x
+        if(!is.null(lambda))
+            x <- BoxCox(x,lambda)
     
     # Choose order of differencing
     if(!is.null(xreg))
@@ -76,20 +87,20 @@ auto.arima <- function(x, d=NA, D=NA, max.p=5, max.q=5,
     }
     else
         offset <- 0
-
+#########################################################################
     if(!stepwise)
     {
-        bestfit <- search.arima(x,d,D,max.p,max.q,max.P,max.Q,max.order,stationary,ic,trace,approximation,xreg=xreg,offset=offset,allowdrift=allowdrift)
-        bestfit$call <- match.call()
-        bestfit$call$x <- data.frame(x=x)
-		bestfit$lamba <- lambda
-		bestfit$x <- orig.x
-		bestfit$series <- deparse(substitute(x))
-		if(!is.null(lambda))
-			bestfit$fitted <- InvBoxCox(bestfit$fitted,lambda)
-        return(bestfit)
+              bestfit <- search.arima(x,d,D,max.p,max.q,max.P,max.Q,max.order,stationary,ic,trace,approximation,xreg=xreg,offset=offset,allowdrift=allowdrift,parallel=parallel, num.cores=num.cores)
+              bestfit$call <- match.call()
+              bestfit$call$x <- data.frame(x=x)
+            bestfit$lamba <- lambda
+            bestfit$x <- orig.x
+            bestfit$series <- deparse(substitute(x))
+            if(!is.null(lambda))
+                bestfit$fitted <- InvBoxCox(bestfit$fitted,lambda)
+              return(bestfit)
     }
-
+#########################################################################
     # Starting model
     p <- start.p <- min(start.p,max.p)
     q <- start.q <- min(start.q,max.q)
@@ -323,14 +334,14 @@ auto.arima <- function(x, d=NA, D=NA, max.p=5, max.q=5,
     }
     
     # Return best fit
-		
+        
     bestfit$x <- orig.x
     bestfit$series <- deparse(substitute(x))
     bestfit$ic <- NULL
     bestfit$call <- match.call()
     bestfit$call$x <- data.frame(x=x)
-	  bestfit$lambda <- lambda
-	  #bestfit$fitted <- fitted(bestfit)
+      bestfit$lambda <- lambda
+      #bestfit$fitted <- fitted(bestfit)
 
     if(trace)
         cat("\n\n Best model:",arima.string(bestfit),"\n\n")
@@ -488,19 +499,19 @@ summary.Arima <- function(object,...)
 # Number of seasonal differences
 nsdiffs <- function(x, m=frequency(x), test=c("ocsb", "ch"))
 {
-	test <- match.arg(test)
-	if(m==1)
-			stop("Non seasonal data")
-	else if(m < 1)
-	{
-			warning("I can't handle data with frequency less than 1. Seasonality will be ignored.")
-			return(0)
-	}
+    test <- match.arg(test)
+    if(m==1)
+            stop("Non seasonal data")
+    else if(m < 1)
+    {
+            warning("I can't handle data with frequency less than 1. Seasonality will be ignored.")
+            return(0)
+    }
 
-	if(test=="ch")
-		return(CHtest(x,m))
-	else
-		return(OCSBtest(x,m))
+    if(test=="ch")
+        return(CHtest(x,m))
+    else
+        return(OCSBtest(x,m))
 
 }
 
@@ -526,73 +537,74 @@ CHtest <- function(x,m)
 # Approximation based on extensive simulations.
 calcOCSBCritVal <- function(seasonal.period) 
 {
-	log.m <- log(seasonal.period)
-	return(-0.2937411*exp(-0.2850853*(log.m-0.7656451)+(-0.05983644)*((log.m-0.7656451)^2))-1.652202)
+    log.m <- log(seasonal.period)
+    return(-0.2937411*exp(-0.2850853*(log.m-0.7656451)+(-0.05983644)*((log.m-0.7656451)^2))-1.652202)
 }
 
 
 OCSBtest <- function(time.series, period) 
 {
-	if(length(time.series) < (2*period+5)) 
-	{
-		return(0)
-	}
-	
-	seas.diff.series <- diff(time.series, lag = period, differences=1)
-	diff.series <- diff(seas.diff.series, lag = 1, differences=1)	
-	
-	y.one <- time.series[2:length(time.series)]
-	y.one <- diff(y.one, lag=period, differences=1)
-	
-	y.two <- time.series[(1+period):length(time.series)]
-	y.two <- diff(y.two, lag=1, differences=1)
-	
-	y.one <- y.one[(1+period):(length(y.one)-1)]
-	y.two <- y.two[2:(length(y.two)-period)]
-	diff.series <- diff.series[(1+period+1):(length(diff.series))]
-	contingent.series <- diff.series
-	
-	x.reg <- cbind(y.one, y.two)
-	diff.series <- ts(data = diff.series, frequency=period)
-	##Turn off warnings
-	old.warning.level <- options()$warn
-	options(warn=-1)
-	regression <- try(Arima(diff.series, order=c(3,0,0), seasonal=list(order=c(1,0,0),period=period), xreg=x.reg), silent=TRUE)
-		
-	if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
-	{
-		regression <- try(Arima(diff.series, order=c(3,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
-			
-		if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
-		{
-			regression <- try(Arima(diff.series, order=c(2,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
-				
-			if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
-			{
-				regression <- try(Arima(diff.series, order=c(1,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
-					
-				if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
-				{
-					regression <- try(lm(contingent.series ~ y.one + y.two - 1, na.action=NULL), silent=TRUE)
-					reg.summary <- summary(regression)
-					t.two <- reg.summary$coefficients[2,3]
-						
-					###Re-enable warnings
-					options(warn=old.warning.level)
-					
-					if((is.nan(t.two)) | (is.infinite(t.two)) | (is.na(t.two)) | (class(regression) == "try-error")) 
-					{
-						return(1)
-					}
-					else
-						return(as.numeric(t.two >= calcOCSBCritVal(period)))
-				}
-			}
-		}
-	} 
+    if(length(time.series) < (2*period+5)) 
+    {
+        return(0)
+    }
+    
+    seas.diff.series <- diff(time.series, lag = period, differences=1)
+    diff.series <- diff(seas.diff.series, lag = 1, differences=1)    
+    
+    y.one <- time.series[2:length(time.series)]
+    y.one <- diff(y.one, lag=period, differences=1)
+    
+    y.two <- time.series[(1+period):length(time.series)]
+    y.two <- diff(y.two, lag=1, differences=1)
+    
+    y.one <- y.one[(1+period):(length(y.one)-1)]
+    y.two <- y.two[2:(length(y.two)-period)]
+    diff.series <- diff.series[(1+period+1):(length(diff.series))]
+    contingent.series <- diff.series
+    
+    x.reg <- cbind(y.one, y.two)
+    diff.series <- ts(data = diff.series, frequency=period)
+    ##Turn off warnings
+    old.warning.level <- options()$warn
+    options(warn=-1)
+    regression <- try(Arima(diff.series, order=c(3,0,0), seasonal=list(order=c(1,0,0),period=period), xreg=x.reg), silent=TRUE)
+        
+    if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
+    {
+        regression <- try(Arima(diff.series, order=c(3,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
+            
+        if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
+        {
+            regression <- try(Arima(diff.series, order=c(2,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
+                
+            if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
+            {
+                regression <- try(Arima(diff.series, order=c(1,0,0), seasonal=list(order=c(0,0,0),period=period), xreg=x.reg), silent=TRUE)
+                    
+                if(class(regression) == "try-error" | tryCatch(any(is.nan(sqrt(diag(regression$var.coef)))), error=function(e) TRUE)) 
+                {
+                    regression <- try(lm(contingent.series ~ y.one + y.two - 1, na.action=NULL), silent=TRUE)
+                    reg.summary <- summary(regression)
+                    t.two <- reg.summary$coefficients[2,3]
+                        
+                    ###Re-enable warnings
+                    options(warn=old.warning.level)
+                    
+                    if((is.nan(t.two)) | (is.infinite(t.two)) | (is.na(t.two)) | (class(regression) == "try-error")) 
+                    {
+                        return(1)
+                    }
+                    else
+                        return(as.numeric(t.two >= calcOCSBCritVal(period)))
+                }
+            }
+        }
+    } 
 
-	se <- sqrt(diag(regression$var.coef))
-	t.two <- regression$coef[names(regression$coef)=="y.two"]/se[names(se)=="y.two"]
-
-	return(as.numeric(t.two >= calcOCSBCritVal(period)))
+    se <- sqrt(diag(regression$var.coef))
+    t.two <- regression$coef[names(regression$coef)=="y.two"]/se[names(se)=="y.two"]
+	###Re-enable warnings
+	options(warn=old.warning.level)
+    return(as.numeric(t.two >= calcOCSBCritVal(period)))
 }
